##############################################################################
#                                                                            #
# Author:   NML_375@Dalnet/ircnet (nml375@kuai.se)                           #
#                                                                            #
# Version:  1.0                                                              #
#                                                                            #
# Date:     970729                                                           #
#                                                                            #
# Desc:     This script will kick users who caps. Capsing is determined by   #
#           percentage caps of whole string... Continuous capsing will also  #
#           result in a ban... (uses toolkit.tcl user-set to store number of #
#           capsing-regulations, user will also be added in userlist with no #
#           flags). The user-record for capsing-regulations are decreased    #
#           every 6 hours.                                                   #
#                                                                            #
# Commands: /msg botnick caps_setlimit [10 - 100 / help]                     #
#               sets max % allowed to be caps (help shows help)              #
#                                                                            #
#           /msg botnick caps_set [1/0]                                      #
#               turns Caps-guard on or off (1 - on, 0 - off)                 #
#                                                                            #
#           /msg botnick caps_status                                         #
#               shows Caps-guard's current settings                          #
#                                                                            #
#           /msg botnick caps_bantime [10 -120 / help]                       #
#               sets # minutes the first ban will last                       #
#                                                                            #
#           /msg botnick caps_clearuser [nick]                               #
#               Manually clears a users record for capsing-regulations       #
#                                                                            #
# Note:     This script is a beta-release and I can't guarantee it's         #
#           functionability or that it won't screw up your eggdrop/system.   #
#           Please e-mail me any bug(s) you might find (suggestions for      #
#           improvements are also welcome :)                                 #
#                                                                            #
##############################################################################

# Modified 970906 by NML_375 (nml375@kuai.se)
#
# Changes since beta:
# * Fixed stupid timer-error in caps_remove
# * No more bugs found so far...

# Things to add:
# * dcc commands
# * multi-channel-support  la eggdrop 1.1

## Version 1.2 by NML_375 (nml375@kuai.se) 970922
# * Added dcc-commands...

putlog "Loading Caps-guard v1.2..."
if {![info exists toolkit_loaded]} {
 catch {source scripts/toolkit.tcl}
 if {![info exists toolkit_loaded]} {
  catch {source toolkit.tcl}
  if {![info exists toolkit_loaded]} {
   error "Unable to load Caps-guard! (Can't find toolkit.tcl)"
  }
 }
}
putlog "Caps-guard: setting trigger to default (30 %)..."
putlog "Caps-guard: setting bantime to default (15 min)..."
set capslimit 30
set caps_set 1
set caps_bantime 15
set caps_ignore 6
bind pubm - * caps_pubm
bind msg - caps_setlimit caps_setlimit
bind msg - caps_set caps_set
bind msg - caps_status caps_status
bind msg - caps_bantime caps_bantime
bind msg - caps_clearuser caps_clearuser

bind dcc - caps caps_main

proc caps_main {hand idx arg} {
 set cmd [lindex $arg 0]
 set args [lrange $arg 1 end]
 switch $cmd {
  "limit" {set r [caps_dcc_setlimit $hand $idx $args]}
  "set" {set r [caps_dcc_set $hand $idx $args]}
  "status" {set r [caps_dcc_status $hand $idx $args]}
  "bantime" {set r [caps_dcc_bantime $hand $idx $args]}
  "clearuser" {set r [caps_dcc_clearuser $hand $idx $args]}
  default {set r 0}
 }
 if {$r == 0} {putdcc $idx "Caps-Guard: invalid command $cmd"}
 return 1
}
proc caps_dcc_setlimit {hand idx text} {
 global capslimit caps_set
 if {!([matchattr $hand m] || [matchattr $hand n])} {return 0}
 if {[string match [string tolower $text] "help"]} {
  putdcc $idx "Syntax: .caps setlimit \[option\]"
  putdcc $idx "Valid options are between 10 and 100"
  return 1
 }
 if {[string trim [lindex $text 0] 0123456789] != ""} {
  putdcc $idx "Please enter a numeric value"
  putdcc $idx "Syntax: .caps setlimit \[value\]"
  return 1
 }
 set caps_temp [lindex $text 0]
 if {!(($caps_temp < 10 ) || ($caps_temp > 100))} {
  putlog "$hand changed Caps-guard's sensitivity from $capslimit to $caps_temp"
  set capslimit $caps_temp
  if {!($caps_set == 1)} {
   putdcc $idx "Warning! Caps-Guard is not activated!"
   putdcc $idx "type .caps set 1 to activate Caps-Guard."
  }
  putdcc $idx "Caps-guard's sensitivity has been set to $capslimit %"
  return 1
 } elseif {($caps_temp < 10) || ($caps_temp > 100)} {
  putdcc $idx "You have entered an invalid option ($caps_temp)."
  putdcc $idx "Valid options are between 10 and 100 or \"help\""
  return 1
 }
}
proc caps_setlimit {nick uhost hand text} {
 global capslimit caps_set botnick
 if {!([matchattr $hand m] || [matchattr $hand n])} {return 0}
 if {[string match [string tolower $text] "help"]} {
  puthelp "NOTICE $nick :Syntax: /msg $botnick caps_setlimit \[option\]"
  puthelp "NOTICE $nick :Valid options are between 10 and 100"
  return 1
 }
 if {[string trim [lindex $text 0] 0123456789] != ""} {
  puthelp "NOTICE $nick :Please enter a numeric value"
  puthelp "NOTICE $nick :Syntax: /msg $botnick caps_setlimit \[value\]"
  return 1
 }
 set caps_temp [lindex $text 0]
 if {!(($caps_temp < 10 ) || ($caps_temp > 100))} {
  putlog "$nick changed Caps-guard's sensitivity from $capslimit to $caps_temp"
  set capslimit $caps_temp
  if {!($caps_set == 1)} {
   puthelp "NOTICE $nick :Warning! Caps-guard is not activated!"
   puthelp "NOTICE $nick :Type /msg $botnick caps_set 1 to activate Caps-guard."
  } else {
   puthelp "NOTICE $nick :Caps-guard's sensitivity has been set to $capslimit %"
  }
  return 1
 } elseif {($caps_temp < 10) || ($caps_temp > 100)} {
  puthelp "NOTICE $nick :You have entered an invalid option ($caps_temp)."
  puthelp "NOTICE $nick :Valid options are between 10 and 100 or \"help\""
  return 1
 }
}

proc caps_dcc_set {hand idx text} {
 global caps_set capslimit caps_bantime botnick
 if {!([matchattr $hand m] || [matchattr $hand n])} {return 0}
 set caps_temp [lindex $text 0]
 if {$caps_temp == 0} {
  set caps_set 0
  putlog "Caps-guard: $hand deactivated Caps-guard"
  putdcc $idx "Caps-guard has been deactivated"
 } elseif {$caps_temp == 1 } {
  set caps_set 1
  putlog "Caps-guard: $hand activated Caps-guard"
  putdcc $idx "Caps-guard has been activated (limit: $capslimit %, ban-time: $caps_bantime min)"
 } else {
  putdcc $idx "You have entered an invalid option ($caps_temp)."
  putdcc $idx "Valid options are: 0 - deactivate, 1 - activate"
 }
 return 1
}

proc caps_set {nick uhost hand text} {
 global caps_set capslimit caps_bantime
 if {!([matchattr $hand m] || [matchattr $hand n])} {return 0}
 set caps_temp [lindex $text 0]
 if {$caps_temp == 0} {
  set caps_set 0
  putlog "Caps-guard: $nick deactivated Caps-guard"
  puthelp "NOTICE $nick :Caps-guard has been deactivated"
 } elseif {$caps_temp == 1 } {
  set caps_set 1
  putlog "Caps-guard: $nick activated Caps-guard"
  puthelp "NOTICE $nick :Caps-guard has been activated (limit: $capslimit %, ban-time: $caps_bantime min)"
 } else {
  puthelp "NOTICE $nick :You have entered an invalid option ($caps_temp)."
  puthelp "NOTICE $nick :Valid options are: 0 - deactivate, 1 - activate"
 }
 return 1
}

proc caps_dcc_status {hand idx text} {
 global caps_set capslimit caps_bantime caps_ignore
 if {!([matchattr $hand m] || [matchattr $hand n])} {return 0}
 putdcc $idx "===== Caps-guard status ====="
 if {$caps_set == 1} {
  putdcc $idx "Caps-guard is : activated"
 } else {
  putdcc $idx "Caps-guard is : de-activated"
 }
 putdcc $idx "Limit         : $capslimit %"
 putdcc $idx "Ban-time      : $caps_bantime min"
 putdcc $idx "Ignoring      : < $caps_ignore chrs"
 putdcc $idx "=====   End of status   ====="
 return 1
}

proc caps_status {nick uhost hand text} {
 global caps_set capslimit caps_bantime botnick caps_ignore
 if {!([matchattr $hand m] || [matchattr $hand n])} {return 0}
 puthelp "NOTICE $nick :===== Caps-guard status ====="
 if {$caps_set == 1} {
  puthelp "NOTICE $nick :Caps-guard is : activated"
 } else {
  puthelp "NOTICE $nick :Caps-guard is : de-activated"
 }
 puthelp "NOTICE $nick :Limit         : $capslimit %"
 puthelp "NOTICE $nick :Ban-time      : $caps_bantime min"
 puthelp "NOTICE $nick :Ignoring      : < $caps_ignore chrs"
 puthelp "NOTICE $nick :=====   End of status   ====="
 return 1
}

proc caps_dcc_bantime {hand idx text} {
 global caps_set caps_limit caps_bantime botnick
 if {!([matchattr $hand m] || [matchattr $hand n])} {
  return 0
 }
 if {[string match [string tolower $text] "help"]} {
  putdcc $idx "Syntax: .caps bantime \[option\]"
  putdcc $idx "Valid options are between 10 and 120"
  return 1
 }
 set caps_temp [lindex $text 0]
 if {[string trim $caps_temp 0123456789] == ""} {
  if {!(($caps_temp < 10 ) || ($caps_temp > 120))} {
   putlog "$hand changed Caps-guard's bantime from $caps_bantime to $caps_temp min"
   set caps_bantime $caps_temp
  } elseif {($caps_temp < 10) || ($caps_temp > 120)} {
   putdcc $idx "You have entered an invalid option ($caps_temp)."
   putdcc $idx "Valid options are between 10 and 120"
   return 1
  }
  putdcc $idx "Caps-guard ban-time is set to $caps_bantime minutes"
 } else {
  putdcc $idx "You have enterred a non-numeric value"
  putdcc $idx "Valid options are between 10 and 120"
 }
 return 1
}

proc caps_bantime {nick uhost hand text} {
 global caps_set caps_limit caps_bantime botnick
 if {!([matchattr $hand m] || [matchattr $hand n])} {
  return 1
 }
 if {[string match [string tolower $text] "help"]} {
  puthelp "NOTICE $nick :Syntax: /msg $botnick caps_bantime \[option\]"
  puthelp "NOTICE $nick :Valid options are between 10 and 120"
  return 1
 }
 set caps_temp [lindex $text 0]
 if {[string trim $caps_temp 0123456789] == ""} {
  if {!(($caps_temp < 10 ) || ($caps_temp > 120))} {
   putlog "$nick changed Caps-guard's bantime from $caps_bantime to $caps_temp min"
   set caps_bantime $caps_temp
  } elseif {($caps_temp < 10) || ($caps_temp > 120)} {
   puthelp "NOTICE $nick :You have entered an invalid option ($caps_temp)."
   puthelp "NOTICE $nick :Valid options are between 10 and 120"
   return 1
  }
  puthelp "NOTICE $nick :Caps-guard ban-time is set to $caps_bantime minutes"
 } else {
  puthelp "NOTICE $nick :You have enterred a non-numeric value"
  puthelp "NOTICE $nick :Valid options are between 10 and 120"
 }
 return 1
}

proc caps_pubm {nick uhost hand chan text} {
 global capslimit caps_set caps_bantime botnick caps_ignore
 if {!($caps_set == 1)} {
  return 0
 }
 if {[matchattr $hand m] == 1 || [matchattr $hand n] == 1 || [matchattr $hand f] == 1 || [matchattr $hand b] == 1} {
  return 0
 }
 if {[matchchanattr $hand m $chan] == 1 || [matchchanattr $hand n $chan] == 1 || [matchchanattr $hand f $chan] == 1 || [matchattr $hand B] == 1} {
  return 0
 }
 if {[string match [string toupper $nick] [string toupper $botnick]]} {
  return 0
 }
 if {![botisop $chan]} {
  return 0
 }
 set caps_upper 0
 set caps_space 0
 set caps_total 0
 set caps_caps 0
 foreach i [split $text {}] {
  if {[string match [string toupper $i] $i] && ![string match [string tolower $i] $i]} {
   incr caps_upper
  }
  if {$i == " "} {
   incr caps_space
  }
  incr caps_total
 }
 set caps_caps [expr [expr 100 * $caps_upper] / [expr $caps_total - $caps_space] ]
 if {$caps_caps > $capslimit } {
  if {([user-get $hand caps] == 0) || ([user-get $hand caps] == "")} {
   if {[expr $caps_total - $caps_space] < $caps_ignore} {
    return 1
   }
   puthelp "PRIVMSG $chan :$nick: Turn Caps-lock off..."
   puthelp "NOTICE $nick :Turn Caps-lock off before you write anything more in $chan ..."
   if {[validuser $hand] == 0} {
    if {[validuser $nick] == 0} {
     adduser $nick *!*[string trimleft [maskhost $uhost] *!]
     chattr $nick ""
     putlog "Caps-guard: Added $nick with no attribs"
     user-set $nick caps 1
     set hand $nick
    } else {
     adduser X$nick *!*[string trimleft [maskhost $uhost] *!]
     chattr X$nick ""
     putlog "Caps-guard: Added X$nick with no attribs"
     user-set X$nick caps 1
     set hand X$nick
    }
   } else {
    user-set $hand caps 1
   }
   timer 360 "caps_remove $hand"
   putlog "Caps-guard: $hand has been warned once"
   return 0
  }
  if {([user-get $hand caps] == 1)} {
   if {[expr $caps_total - $caps_space] < 6} {
    return 1
   }
   putserv "KICK $chan $nick :I told you to turn Caps-lock off!"
   user-set $hand caps 2
   timer 360 "caps_remove $hand"
   putlog "Caps-guard: $nick has been warned a second time (using kick...)"
   return 0
  }
  if {([user-get $hand caps] == 2)} {
   if {[expr $caps_total - $caps_space] < 6} {
    return 1
   }
   putserv "MODE $chan -o $nick"
   newchanban $chan *!*[string trimleft [maskhost $uhost] *!] CapsGuard "Capsing..." $caps_bantime
   putserv "KICK $chan $nick :I've already warned you twice, now I'm banning you for $caps_bantime minutes (capsing)"
   user-set $hand caps 3
   timer 360 "caps_remove $hand"
   putlog "Caps-guard: $nick has been banned in $caps_bantime minutes (*!*[string trimleft [maskhost $uhost] *!])"
   return 0
  }
  if {([user-get $hand caps] == 3)} {
   if {[expr $caps_total - $caps_space] < 6} {
    return 1
   }
   putserv "MODE $chan -o :$nick"
   newchanban $chan *!*[string trimleft [maskhost $uhost] *!] CapsGuard "Capsing... (Perm-banned!)" 0
   putserv "KICK $chan $nick :You are not welcome here anymore! (Capsing)"
   user-set $hand caps 4
   putlog "Caps-guard: $nick has been perm-banned (*!*[string trimleft [maskhost $uhost] *!])"
  }
 }
}
proc caps_remove {hand} {
 if {([user-get $hand caps] == 0) || ([user-get $hand caps] == 4)} {
  return 0
 } elseif {([user-get $hand caps] == "") || ([user-get $hand caps] == 1)} {
  user-set $hand caps 0
  return 2
 } else {
  user-set $hand caps [expr [user-get $hand caps] - 1]
  timer 360 "caps_remove $hand"
  return 1
 }
}
proc caps_dcc_clearuser {hand idx text} {
 if {!([matchattr $hand m] || [matchattr $hand n])} {return 0}
 if {[validuser [lindex $text 0]] == 1} {
  if {[user-get [lindex $text 0] caps] == 0} {
   putdcc $idx "[lindex $text 0]'s record for caps-regulations has already been cleared..."
   return 1
  } else {
   user-set [lindex $text 0] caps 0
   putdcc $idx "[lindex $text 0]'s record for caps-regulations has been cleared..."
   putlog "Caps-Guard: $hand has cleared [lindex $text 0]'s record for caps-regulations"
  }
 } else {
  putdcc $idx "Sorry, but I don't know who [lindex $text 0] is..."
  return 1
 }
}

proc caps_clearuser {nick uhost hand text} {
 if {!([matchattr $hand m] || [matchattr $hand n])} {return 1}
 if {[validuser [lindex $text 0]] == 1} {
  if {[user-get [lindex $text 0] caps] == 0} {
   puthelp "NOTICE $nick :[lindex $text 0]'s record for caps-regulations has already been cleared..."
   return 1
  } else {
   user-set [lindex $text 0] caps 0
   puthelp "NOTICE $nick :[lindex $text 0]'s record for caps-regulations has been cleared..."
   putlog "CapsGuard: $nick (handle: $hand) has cleared [lindex $text 0]'s record for caps-regulations"
  }
 } else {
  puthelp "NOTICE $nick :Sorry, but I don't know who [lindex $text 0] is..."
  return 1
 }
}
putlog "Caps-guard v1.2 Loaded!   Author: NML_375 (nml375@kuai.se))"
